package ni2.likewind.image;
/*
 * ImageCreator
 * 
 * Copyright (c) 2001, 2002, 2003 Marco Schmidt.
 * All rights reserved.
 * 
 * 이 클래스는 jiu 패키지에 있던 net.sourceforge.jiu.gui.awt.ImageCreator 클래스를
 * AWT Image 대신 SWT Image를 사용하도록 약간 수정한 것입니다.
 * RGB24Image 외에는 아직 제대로 수정을 가하지 않았습니다.
 * 자세한 변경 내용은 diff등의 툴을 사용하세요.
 * -ni2 (netisinfinite@gmail.com), 06.09.26 v0.1
 */

import net.sourceforge.jiu.data.BilevelImage;
import net.sourceforge.jiu.data.Gray16Image;
import net.sourceforge.jiu.data.Gray8Image;
import net.sourceforge.jiu.data.Palette;
import net.sourceforge.jiu.data.Paletted8Image;
import net.sourceforge.jiu.data.PixelImage;
import net.sourceforge.jiu.data.RGB24Image;
import net.sourceforge.jiu.data.RGB48Image;
import net.sourceforge.jiu.data.RGBIndex;
import net.sourceforge.jiu.gui.awt.RGBA;

import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;

/**
 * A class to create java.awt.Image objects from various JIU image data types
 * and vice versa.
 * java.awt.Image objects can be used with the AWT and Swing GUI environments.
 *
 * @author Marco Schmidt
 */
public class ImageCreator
{
	/**
	 * The default transparency value to be used: full opacity.
	 */
	public static final int DEFAULT_ALPHA = 0xff000000;
	private static final int scanlinepad = 1;
	private static Device parent;

	private ImageCreator()
	{
	}

	/**
	 * Creates a {@link java.awt.Image} object from a pixel array.
	 * Internally, a {@link java.awt.Frame} object is used to call its 
	 * {@link java.awt.Frame#createImage} method
	 * with a {@link java.awt.image.MemoryImageSource} object.
	 *
	 * @param pixels the image pixel data in the typical RGBA 32-bit format, one int per pixel
	 * @param width the horizontal resolution in pixels of the image to be created
	 * @param height the vertical resolution in pixels of the image to be created
	 */
	public static Image createImage(int[] pixels, int width, int height, Device parent)
	{
		ImageCreator.parent = parent;
		if (width < 1 || height < 1)
		{
			throw new IllegalArgumentException("Error -- width and height " +
				"must both be larger than zero.");
		}
		if (pixels == null)
		{
			throw new IllegalArgumentException("Error -- the pixel array " +
				"must be non-null.");
		}
		if (pixels.length < width * height)
		{
			throw new IllegalArgumentException("Error -- the pixel array " +
				"must contain at least width times height items.");
		}
		return new Image(parent,new ImageData(width, height, parent.getDepth(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff)));
	}

	/**
	 * Creates an instance of {@link java.awt.Image} from an instance of
	 * {@link RGB24Image}.
	 * This will require <code>image.getWidth() * image.getHeight() * 4</code>
	 * bytes of free memory.
	 * This method checks the type of the argument image via instanceof
	 * and the calls the right convertToSwtImage method of this class.
	 * @param image the RGB24Image to be converted
	 * @return newly-created SWT image instance
	 */
	public static Image convertToSwtImage(PixelImage image, int alpha)
	{
		if (image == null)
		{
			System.out.println("image == null");
			return null;
		}
		if (image instanceof RGB24Image)
		{
			System.out.println("image instanceof RGB24Image");
			return convertToSwtImage((RGB24Image)image, alpha);
		}
		else
		if (image instanceof RGB48Image)
		{
			System.out.println("image instanceof RGB48Image");
			return convertToSwtImage((RGB48Image)image, alpha);
		}
		else
		if (image instanceof Gray8Image)
		{
			System.out.println("image instanceof Gray8Image");
			return convertToSwtImage((Gray8Image)image, alpha);
		}
		else
		if (image instanceof Gray16Image)
		{
			System.out.println("image instanceof Gray16Image");
			return convertToSwtImage((Gray16Image)image, alpha);
		}
		else
		if (image instanceof Paletted8Image)
		{
			System.out.println("image instanceof Paletted8Image");
			return convertToSwtImage((Paletted8Image)image, alpha);
		}
		else
		if (image instanceof BilevelImage)
		{
			System.out.println("image instanceof BilevelImage");
			return convertToSwtImage((BilevelImage)image, alpha);
		}
		else
		{
			return null;
		}
	}

	/**
	 * Convert a BilevelImage object to an SWT image object.
	 * @param image the image to be converted
	 * @param alpha the transparency value to be written to each
	 *  pixel in the resulting image
	 * @return newly-created SWT image
	 */
	public static Image convertToSwtImage(BilevelImage image, int alpha)
	{
		if (image == null)
		{
			return null;
		}
		int width = image.getWidth();
		int height = image.getHeight();
		if (width < 1 || height < 1)
		{
			return null;
		}
		int bytesPerRow = (width + 7) / 8;
		int[] pixels = new int[width * height];
		byte[] row = new byte[bytesPerRow];
		int destOffset = 0;
		for (int y = 0; y < height; y++)
		{
			image.getPackedBytes(0, y, width, row, 0, 0);
			RGBA.convertFromPackedBilevel(row, 0, alpha, pixels, destOffset, width);
			destOffset += width;
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[4*i] = (byte)(pixels[i] >> 24 & 0xff);
			byteData[4*i+1] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[4*i+2] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[4*i+3] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
	}

	/**
	 * Creates an SWT Image object from a Gray16Image object and an alpha value.
	 * This is done by allocating a new int array with image.getWidth() times
	 * image.getHeight() elements, copying the data to those ints (using transparency
	 * information from the top eight bits of the alpha argument) and calling
	 * Toolkit.createImage with a MemoryImageSource of those int[] pixels.
	 * @param image the grayscale image to be converted
	 * @param alpha the alpha value, bits must only be set in the top eight bits
	 * @return SWT image created from the argument input image
	 */
	public static Image convertToSwtImage(Gray16Image image, int alpha)
	{
		if (image == null)
		{
			return null;
		}
		int width = image.getWidth();
		int height = image.getHeight();
		if (width < 1 || height < 1)
		{
			return null;
		}
		int[] pixels = new int[width * height];
		short[] gray = new short[width];
		int destOffset = 0;
		for (int y = 0; y < height; y++)
		{
			image.getShortSamples(0, 0, y, width, 1, gray, 0);
			RGBA.convertFromGray16(gray, 0, alpha, pixels, destOffset, width);
			destOffset += width;
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[4*i] = (byte)(pixels[i] >> 24 & 0xff);
			byteData[4*i+1] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[4*i+2] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[4*i+3] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
	}

	/**
	 * Creates an AWT Image object from a Gray8Image object and an alpha value.
	 * This is done by allocating a new int array with image.getWidth() times
	 * image.getHeight() elements, copying the data to those ints (using transparency
	 * information from the top eight bits of the alpha argument) and calling
	 * Toolkit.createImage with a MemoryImageSource of those int[] pixels.
	 *
	 * @param image the grayscale image to be converted
	 * @param alpha the alpha value, bits must only be set in the top eight bits
	 * @return SWT image created from the argument input image
	 */
	public static Image convertToSwtImage(Gray8Image image, int alpha)
	{
		if (image == null)
		{
			return null;
		}
		int width = image.getWidth();
		int height = image.getHeight();
		if (width < 1 || height < 1)
		{
			return null;
		}
		int[] pixels = new int[width * height];
		byte[] gray = new byte[width];
		int destOffset = 0;
		for (int y = 0; y < height; y++)
		{
			image.getByteSamples(0, 0, y, width, 1, gray, 0);
			RGBA.convertFromGray8(gray, 0, alpha, pixels, destOffset, width);
			destOffset += width;
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[4*i] = (byte)(pixels[i] >> 24 & 0xff);
			byteData[4*i+1] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[4*i+2] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[4*i+3] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
	}

	public static Image convertToSwtImage(Paletted8Image image, int alpha)
	{
		if (image == null)
		{
			return null;
		}
		int width = image.getWidth();
		int height = image.getHeight();
		Palette palette = image.getPalette();
		if (width < 1 || height < 1 || palette == null)
		{
			return null;
		}
		int[] red = new int[palette.getNumEntries()];
		int[] green = new int[palette.getNumEntries()];
		int[] blue = new int[palette.getNumEntries()];
		for (int i = 0; i < palette.getNumEntries(); i++)
		{
			red[i] = palette.getSample(RGBIndex.INDEX_RED, i);
			green[i] = palette.getSample(RGBIndex.INDEX_GREEN, i);
			blue[i] = palette.getSample(RGBIndex.INDEX_BLUE, i);
		}
		int[] pixels = new int[width * height];
		byte[] data = new byte[width];
		int destOffset = 0;
		for (int y = 0; y < height; y++)
		{
			image.getByteSamples(0, 0, y, width, 1, data, 0);
			RGBA.convertFromPaletted8(data, 0, alpha, red, green, blue, pixels, destOffset, width);
			destOffset += width;
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[4*i] = (byte)(pixels[i] >> 24 & 0xff);
			byteData[4*i+1] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[4*i+2] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[4*i+3] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
		
	}

	public static Image convertToSwtImage(RGB24Image image, int alpha)
	{
		if (image == null)
		{
			return null;
		}
		int width = image.getWidth();
		int height = image.getHeight();
		if (width < 1 || height < 1)
		{
			return null;
		}
		int[] pixels = new int[width * height];
		byte[] red = new byte[width];
		byte[] green = new byte[width];
		byte[] blue = new byte[width];
		int destOffset = 0;
		for (int y = 0; y < height; y++)
		{
			image.getByteSamples(RGBIndex.INDEX_RED, 0, y, width, 1, red, 0);
			image.getByteSamples(RGBIndex.INDEX_GREEN, 0, y, width, 1, green, 0);
			image.getByteSamples(RGBIndex.INDEX_BLUE, 0, y, width, 1, blue, 0);
			RGBA.convertFromRGB24(red, 0, green, 0, blue, 0, alpha, pixels, destOffset, width);
			destOffset += width;
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[3*i] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[3*i+1] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[3*i+2] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
		
	}

	public static Image convertToSwtImage(RGB48Image image, int alpha)
	{
		if (image == null)
		{
			return null;
		}
		int width = image.getWidth();
		int height = image.getHeight();
		if (width < 1 || height < 1)
		{
			return null;
		}
		int[] pixels = new int[width * height];
		short[] red = new short[width];
		short[] green = new short[width];
		short[] blue = new short[width];
		int destOffset = 0;
		for (int y = 0; y < height; y++)
		{
			image.getShortSamples(RGBIndex.INDEX_RED, 0, y, width, 1, red, 0);
			image.getShortSamples(RGBIndex.INDEX_GREEN, 0, y, width, 1, green, 0);
			image.getShortSamples(RGBIndex.INDEX_BLUE, 0, y, width, 1, blue, 0);
			RGBA.convertFromRGB48(red, 0, green, 0, blue, 0, alpha, pixels, destOffset, width);
			destOffset += width;
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[4*i] = (byte)(pixels[i] >> 24 & 0xff);
			byteData[4*i+1] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[4*i+2] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[4*i+3] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
	}

	/**
	 * Creates an {@link RGB24Image} from the argument SWT image instance.
	 * @param image SWT image object to be converted to a {@link RGB24Image}
	 * @return a {@link RGB24Image} object holding the image data from the argument image
	 */
	/*
	public static RGB24Image convertImageToRGB24Image(Image image)
	{
		//PixelGrabber때문에 일단 보류
		if (image == null)
		{
			return null;
		}
		ImageData imageData = image.getImageData();
		int width = imageData.width;
		int height = imageData.width;
		if (width < 1 || height < 1)
		{
			return null;
		}
		int[] pixels = new int[width * height];
		java.awt.image.PixelGrabber pg = new java.awt.image.PixelGrabber(image, 0, 0, width, height, pixels, 0, width);
		try
		{
			pg.grabPixels();
		}
		catch (InterruptedException e)
		{
			return null;
		}
		if ((pg.getStatus() & java.awt.image.ImageObserver.ABORT) != 0)
		{
			return null;
		}
		RGB24Image result = new MemoryRGB24Image(width, height);
		int offset = 0;
		for (int y = 0; y < height; y++)
		{
			for (int x = 0; x < width; x++)
			{
				int pixel = pixels[offset++] & 0xffffff;
				// TODO: store alpha value; requires some sort of 
				// transparency channel data type yet to be implemented
				result.putSample(RGBIndex.INDEX_RED, x, y, pixel >> 16);
				result.putSample(RGBIndex.INDEX_GREEN, x, y, (pixel >> 8) & 0xff);
				result.putSample(RGBIndex.INDEX_BLUE, x, y, pixel & 0xff);
			}
		}
		byte[] byteData = new byte[pixels.length * 4];
		for (int i = 0; i < pixels.length; i++) {
			byteData[4*i] = (byte)(pixels[i] >> 24 & 0xff);
			byteData[4*i+1] = (byte)(pixels[i] >> 16 & 0xff);
			byteData[4*i+2] = (byte)(pixels[i] >> 8 & 0xff);
			byteData[4*i+3] = (byte)(pixels[i] & 0xff);
		}
		return new Image(parent,new ImageData(width, height, image.getBitsPerPixel(), new PaletteData(0x00ff0000, 0x0000ff00, 0x000000ff), scanlinepad, byteData));
	}
	*/
}
