package ni2.likewind.image;

import java.io.IOException;

import net.sourceforge.jiu.codecs.InvalidFileStructureException;
import net.sourceforge.jiu.codecs.InvalidImageIndexException;
import net.sourceforge.jiu.codecs.UnsupportedTypeException;
import net.sourceforge.jiu.data.PixelImage;
import net.sourceforge.jiu.gui.awt.RGBA;
import ni2.likewind.io.SVGImageLoader;
import ni2.likewind.io.VFSImageLoader;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

/**
 * 이미지를 불러오는 클래스.
 * SWT를 이용한 이미지 로딩은 native를 사용해 빠르다. JIU는 많이 느리기 때문에
 * 되도록 SWT를 사용한다. 
 * 
 * @author ni2 (netisinfinite@gmail.com)
 * @version 0.1
 * This code licensed under the terms of the GPL.  See doc/LICENSE.
 */
public class ImageLoader {
	//TODO 시작 성능을 떨어뜨리지 않으면서도 통합된 지원 파일 관리 방법이 필요...
	public static final String[] SWT_FILE_TYPES = {
		"bmp","gif","ico","jpeg","jpg","png","rle"
	};
	public static final String[] JIU_FILE_TYPES = {
		"pbm", "png", "gif", "ras", "pcd", "tiff", "bmp", "iff", "lbm", "psd"
	};
	/*
	 * 현재의 vfs에서는 압축되지 않은 tgz, tbz2만 지원하기 때문에 지원 목록에서 제외했다.
	 * 자세한 내용은 http://jakarta.apache.org/commons/vfs/filesystems.html
	 */
	public static final String[] ARCHIVE_FILE_TYPES = {
		"jar", "zip", "tar"
	};
	public static final String[] SUPPORTED_FILE_TYPES = {
		"bmp","gif","ico","iff","jpeg","jpg","lbm","pbm","pcd","pgm","png","pnm","ppm","psd","ras","rle","tif","tiff"
	};
	public static final String[] SUPPORTED_FILE_TYPES_EXT = {
		"*.bmp;*.gif;*.ico;*.iff;*.jpeg;*.jpg;*.lbm;*.pbm;*.pcd;*.pgm;*.png;*.pnm;*.ppm;*.psd;*.ras;*.rle;*.tif;*.tiff",
		"*.bmp","*.gif","*.ico","*.iff","*.jpeg","*.jpg","*.lbm","*.pbm","*.pcd","*.pgm","*.png","*.pnm","*.ppm","*.psd","*.ras","*.rle","*.tif","*.tiff"
	};
	
	/**
	 * 주어진 파일명으로부터 SWT 이미지 객체를 생성한다.
	 * @param fileName 불러 올 파일의 이름.
	 * @return 이미지 파일로부터 생성한 Image 객체를 반환하며 실패시 null을 반환한다.
	 * @throws InvalidFileStructureException
	 * @throws InvalidImageIndexException
	 * @throws UnsupportedTypeException
	 * @throws IOException
	 */
	public static Image loadImage(String fileName){
		Image image = loadFromSWT(fileName);
		if(image == null){
			image = loadFromJIU(fileName);
		}
		if(image == null && archiveSupport){
			image = loadFromArchive(fileName);
		}
		if(image == null && svgSupport){
			image = loadFromSVG(fileName);
		}
		return image;
	}
	
	/* 
	SWT supported type(ImageData.type) :
	IMAGE_BMP
	    Windows BMP file format, no compression
	IMAGE_BMP_RLE
	    Windows BMP file format, RLE compression if appropriate
	IMAGE_GIF
	    GIF file format
	IMAGE_ICO
	    Windows ICO file format
	IMAGE_JPEG
	    JPEG file format
	IMAGE_PNG
	    PNG file format 
	*/
	private static final Display device = Display.getCurrent();
	public static Image loadFromSWT(String fileName){
		Image image = null;
		
		if(canLoadFromSWT(fileName))
				/*
				 * Image(device, string)은 native를 이용하기 때문에
				 * 현재(3235) 윈도우, 리눅스에서 다른 방식의 생성자보다 빠르다. 
				 * SWT Image를 불러들이는 속도는 다음 페이지가 도움이 될 것이다.
				 * https://bugs.eclipse.org/bugs/show_bug.cgi?id=24451#c51
				 */
				image = new Image(device, fileName);
		
		return image;
	}
	
	public static boolean canLoadFromSWT(String fileName){
		String lowerFileName = fileName.toLowerCase();
		
		for(String ext : SWT_FILE_TYPES){
			if(lowerFileName.endsWith(ext)){
				return true;
			}
		}
		return false;
	}
	
	public static Image loadFromJIU(String fileName){
		Image image = null;
		try {
			PixelImage pi = net.sourceforge.jiu.codecs.ImageLoader.load(fileName);
			if(pi != null)
				image = ImageCreator.convertToSwtImage(pi, RGBA.DEFAULT_ALPHA);
		} catch (Exception e){
			e.printStackTrace();
		}
		
		return image;
	}
	

	/*
	 * 아래 변수들은 libx 확장을 통해 지원되는 기능들의 실행 여부를 결정한다.
	 * 기본값은 true이며, false로 변경될 경우 읽기를 다시 시도하지 않는다.
	 */
	private static boolean archiveSupport = true;
	private static boolean svgSupport = true;
	
	/*
	 * 아래의 메서드들은 libx 확장을 통해 지원되는 기능들이다.
	 * libx 확장 미설치 상태에서 실행시 NoClassDefFoundError가 나는 것을 방지하기 위해 클래스를 분리했다.
	 * TODO "확장 설치가 필요하다"는 메시지를 일관되게 처리할 수 있는 메서드가 필요.
	 */
	public static Image loadFromArchive(String fileName){
		Image image = null;
		String lowerFileName = fileName.toLowerCase();
		for(String ext : ARCHIVE_FILE_TYPES){
			if(lowerFileName.endsWith(ext)){
				/*
				 * 실행시마다 Exception 객체가 생성되는 것을 막기 위해 별도 메서드화 
				 */
				image = loadFromArchive(fileName, ext);
			}
		}
		return image;
	}
	
	private static Image loadFromArchive(String fileName, String ext){
		Image image = null;
		try{
			image = new VFSImageLoader().loadFromArchive(device, fileName, ext);
		}catch(NoClassDefFoundError e){
			archiveSupport = false;
			System.out.println("ARCHIVE 지원 확장이 필요합니다.");
		}
		return image;
	}
	
	public static Image loadFromSVG(String fileName){
		Image image = null;
		try{
			image = new SVGImageLoader().load(device, fileName);
		}catch(NoClassDefFoundError e){
			svgSupport = false;
			System.out.println("SVG 지원 확장이 필요합니다.");
		}
		return image;
	}
}
